﻿using System.ComponentModel;
using UserStageExample.FlexiService;

namespace UserStageExample.ViewModel
{
    /// <summary>
    /// Possible document states
    /// </summary>
    internal enum DocumentState
    {
        /// <summary>
        /// Not selected
        /// </summary>
        Untagged,

        /// <summary>
        /// Document confirmed
        /// </summary>
        Accepted,

        /// <summary>
        /// Document rejected
        /// </summary>
        Rejected
    }

    /// <summary>
    /// Document controller
    /// </summary>
    internal class DocumentViewModel : INotifyPropertyChanged
    {
        /// <summary>
        /// Displayed document
        /// </summary>
        private Document _document;
        /// <summary>
        /// Document state
        /// </summary>
        private DocumentState _state = DocumentState.Untagged;
        /// <summary>
        /// Appears when properties of a document controlled change
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;
        /// <summary>
        /// Document ID
        /// </summary>
        public int Id { get; private set; }
        /// <summary>
        /// Source document
        /// </summary>
        public Document Document
        {
            get { return _document; }
        }
        /// <summary>
        /// A task that a document belongs to
        /// </summary>
        public TaskViewModel Task { get; set; }
        /// <summary>
        /// Document name
        /// </summary>
        public string Name
        {
            get { return _document == null ? null : _document.PersonName(); }
        }
        /// <summary>
        /// Determines whether a document was accepted
        /// </summary>
        public bool IsAccepted { get { return _state == DocumentState.Accepted; } }
        /// <summary>
        /// Determines whether a document was rejected
        /// </summary>
        public bool IsRejected { get { return _state == DocumentState.Rejected; } }

        /// <summary>
        /// Document state
        /// </summary>
        public DocumentState State
        {
            get { return _state; }
            set
            {
                _state = value;
                OnPropertyChanged("State");
                OnPropertyChanged("IsAccepted");
                OnPropertyChanged("IsRejected");
            }
        }

        /// <summary>
        /// Creates new document based on a model
        /// </summary>
        /// <param name="id">Document ID</param>
        /// <param name="document">Document model</param>
        public DocumentViewModel(int id, Document document)
        {
            Id = id;
            _document = document;
        }

        /// <summary>
        /// Updates document controller values with model values
        /// </summary>
        /// <param name="document">Document model</param>
        public void Update(Document document)
        {
            var oldName = Name;
            _document = document;
            if (oldName != Name) OnPropertyChanged("Name");
        }

        /// <summary>
        /// Confirmes a document or cancels the confirmation
        /// </summary>
        public void Accept()
        {
            State = State == DocumentState.Accepted ? DocumentState.Untagged : DocumentState.Accepted;
        }

        /// <summary>
        /// Rejects a document or confirms the rejection
        /// </summary>
        public void Reject()
        {
            State = State == DocumentState.Rejected ? DocumentState.Untagged : DocumentState.Rejected;
        }

        /// <summary>
        /// Forms the PropertyChanged message
        /// </summary>
        /// <param name="propertyName">Property name</param>
        protected virtual void OnPropertyChanged(string propertyName)
        {
            var h = PropertyChanged;
            if (h != null) h.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}