﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using UserStageExample.FlexiService;

namespace UserStageExample.ViewModel
{
    /// <summary>
    /// Main window mode
    /// </summary>
    internal enum WindowMode
    {
        /// <summary>
        /// Login form
        /// </summary>
        Auth,
        /// <summary>
        /// Waiting area
        /// </summary>
        Waiting,
        /// <summary>
        /// General view where tasks are displayed
        /// </summary>
        Loaded
    }

    /// <summary>
    /// Main controller of the app
    /// </summary>
    internal class ServiceViewModel : INotifyPropertyChanged
    {
        /// <summary>
        /// Current main window mode
        /// </summary>
        private WindowMode _mode = WindowMode.Auth;

        /// <summary>
        /// Separate authorisation controller
        /// </summary>
        private AuthViewModel _auth = new AuthViewModel();

        /// <summary>
        /// Notification that shows that a property of the controller was modified
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Update delay
        /// </summary>
        private DateTime _updateDelay = DateTime.MinValue;

        /// <summary>
        /// Creates a new app controlled
        /// </summary>
        public ServiceViewModel()
        {
            Tasks = new ObservableCollection<TaskViewModel>();
        }

        /// <summary>
        /// A list of processed tasks
        /// </summary>
        public ObservableCollection<TaskViewModel> Tasks { get; private set; }

        /// <summary>
        /// Authentication collector
        /// </summary>
        public AuthViewModel Auth
        {
            get { return _auth; }
            private set { _auth = value; OnPropertyChanged(); }
        }

        /// <summary>
        /// Main window mode
        /// </summary>
        public WindowMode Mode
        {
            get { return _mode; }
            private set { _mode = value; OnPropertyChanged(); }
        }

        /// <summary>
        /// Calls the PropertyChanged event
        /// </summary>
        /// <param name="propertyName">Name of the modified property</param>
        protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            var h = PropertyChanged;
            if (h != null) h.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        /// <summary>
        /// Updates a set of tasks based on the server data
        /// </summary>
        /// <param name="tasks">List of tasks received from the server</param>
        public void UpdateTasks(Dictionary<int, Task> tasks)
        {
            // If an update is delayed, new tasks are ignored
            if (_updateDelay > DateTime.Now) return;
            // Updating current tasks and removing lost ones
            for (var i = 0; i < Tasks.Count; i++)
            {
                var taskViewModel = Tasks[i];
                Task task;
                if (tasks.TryGetValue(taskViewModel.Id, out task))
                {
                    tasks.Remove(taskViewModel.Id);
                    taskViewModel.Update(task);
                }
                else
                {
                    Tasks.RemoveAt(i--);
                }
            }
            // Adding new tasks
            foreach (var task in tasks)
            {
                Tasks.Add(new TaskViewModel(task.Key, task.Value));
            }
            Mode = WindowMode.Loaded;
        }

        /// <summary>
        /// Rejects a document
        /// </summary>
        /// <param name="document">Rejected document</param>
        /// <returns>Shows whether a task is ready to be sent</returns>
        public bool Reject(DocumentViewModel document)
        {
            document.Reject();
            return UpdateTask(document.Task);
        }

        /// <summary>
        /// Confirms a document
        /// </summary>
        /// <param name="document">Confirmed document</param>
        /// <returns>Shows whether a task is ready to be sent</returns>
        public bool Accept(DocumentViewModel document)
        {
            document.Accept();
            return UpdateTask(document.Task);
        }

        /// <summary>
        /// Updates a task
        /// </summary>
        /// <param name="task">Updated task</param>
        /// <returns>Shows whether a task is ready to be sent</returns>
        private bool UpdateTask(TaskViewModel task)
        {
            if (task.HasUntagged) return false;
            Tasks.Remove(task);
            // The form update is stopped for a few seconds 
            // so that the closed task does not appear again.
            _updateDelay = DateTime.Now.AddSeconds(3);
            return true;
        }

        /// <summary>
        /// Switches a window to the waiting mode
        /// </summary>
        public void WaitLoading()
        {
            Mode = WindowMode.Waiting;
        }

        /// <summary>
        /// Returns a window to the authentication mode and sets the error message
        /// </summary>
        /// <param name="message">Error message</param>
        public void RevertAuth(string message)
        {
            Auth.LoginResult = message;
            Mode = WindowMode.Auth;
        }
    }
}
