﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using UserStageExample.FlexiService;

namespace UserStageExample.ViewModel
{
    /// <summary>
    /// Shows a FlexiCapture task
    /// </summary>
    internal class TaskViewModel
    {
        /// <summary>
        /// The ID a FlexiCapture task
        /// </summary>
        public int Id { get; private set; }
        /// <summary>
        /// A FlexiCapture batch ID that includes documents of the task
        /// </summary>
        public int BatchId { get; private set; }
        /// <summary>
        /// A set of controlled documents.
        /// </summary>
        public ObservableCollection<DocumentViewModel> Documents { get; private set; }
        /// <summary>
        /// Determines whether the task contains untagged documents
        /// </summary>
        public bool HasUntagged { get { return Documents.Any(x => x.State == DocumentState.Untagged); } }

        /// <summary>
        /// Creates a new task controller based on a task model
        /// </summary>
        /// <param name="id">Task ID</param>
        /// <param name="task">Task model</param>
        public TaskViewModel(int id, Task task)
        {
            Id = id;
            BatchId = task.BatchId;
            Documents = new ObservableCollection<DocumentViewModel>();
            UpdateDocuments(task.Documents);
        }

        /// <summary>
        /// Updatesthe task controller with values from the received model
        /// </summary>
        /// <param name="task">Task model</param>
        public void Update(Task task)
        {
            UpdateDocuments(task.Documents);
        }

        /// <summary>
        /// Updates the list of task documents based on the receive list
        /// </summary>
        /// <remarks>
        /// This method compares displayed document IDs with received IDs
        /// If the document displays, than it is updated, if it does not display, it is added.
        /// If the displayed document is not sent, it is removed.
        /// </remarks>
        /// <param name="documents">Required document list</param>
        private void UpdateDocuments(IDictionary<int, Document> documents)
        {
            // Updating existing and removind lost documents
            for (var i = 0; i < Documents.Count; i++)
            {
                var documentViewModel = Documents[i];
                Document document;
                if (documents.TryGetValue(documentViewModel.Id, out document))
                {
                    documents.Remove(documentViewModel.Id);
                    documentViewModel.Update(document);
                }
                else
                {
                    Documents.RemoveAt(i--);
                }
            }
            // Adding new documents
            foreach (var document in documents)
            {
                Add(new DocumentViewModel(document.Key, document.Value));
            }
        }

        /// <summary>
        /// Adds a new document
        /// </summary>
        /// <param name="documentViewModel"></param>
        private void Add(DocumentViewModel documentViewModel)
        {
            Documents.Add(documentViewModel);
            documentViewModel.Task = this;
        }
    }
}
